import pytest
import json

# Only going to import the interesting bits
from jobtriggers import cloud_compose_complete_msg
from jobtriggers import exceptions as exc

# Set up some dummy data to work with that's in various states of correct
# These comprise just the interesting bits from the fedmsg that we want to
# consume with our trigger.

template_msg = ('{"msg": { "status": "FINISHED",'
                '"location":'
                '"https://kojipkgs.fedoraproject.org/compose/Fedora-Cloud-25-20170313.0/compose",'
                '"compose_id": ""}}')

msg_good = json.loads(template_msg)
msg_good['msg']['compose_id'] = "Fedora-Cloud-24-20170302.0"

msg_bad_compose_id = json.loads(template_msg)
msg_bad_compose_id['msg']['compose_id'] = "Fedora-Cloud-23"

msg_atomic = json.loads(template_msg)
msg_atomic['msg']['compose_id'] = "Fedora-Atomic-25-20170302.0"


@pytest.fixture(autouse=True)
def _no_requests(monkeypatch):
    monkeypatch.setattr(cloud_compose_complete_msg,
                        "_fetch_metadata",
                        _fetch_metadata)


@pytest.fixture(scope="module")
def _fetch_metadata(compose_url):
    """This is a stub to provide mocked requests objects so we don't have
    to hit the network in our tests."""

    # This is really ugly, but it's as sparse a dataset as we need to
    # complete the test.
    dummy_data = ('{"payload":'
                      '{"images":'
                          '{"CloudImages":'
                              '{"x86_64": ['
                                   '{"format": "qcow2",'
                                    '"path": "/test/path/image.qcow2"'
                                    '}]'
                              '}'
                          '}'
                      '}'
                  '}')

    return json.loads(dummy_data)


def test_process():
    """Test that nothing fails with known good data."""

    data = cloud_compose_complete_msg._process(msg_good)
    assert data['message_type'] == 'CloudCompose'
    assert data['name'] == 'Fedora-Cloud'
    assert data['version'] == 'Cloud'
    assert data['release'] == '24'


def test_process_atomic():
    """Test that the detection of Atomic composes sets the correct
    message_type, which would be "AtomicCompose"."""

    data = cloud_compose_complete_msg._process(msg_atomic)

    assert data['name'] == "Fedora-Atomic"
    assert data['version'] == "Atomic"
    assert data['message_type'] == "AtomicCompose"


def test_pull_version_release():
    """Ensure that execution failes if the 'compose_id' is malformed."""
    with pytest.raises(exc.TriggerMsgError):
        cloud_compose_complete_msg._process(msg_bad_compose_id)
