import pytest
import mock
from dingus import Dingus
from munch import Munch

from jobtriggers import pagure_git_received
from jobtriggers import utils


def mock_clone_repo(repo, branch='master', fallback=None):
    return '/tmp/path_to_repo', branch


@pytest.mark.usefixtures('prepare')
class TestPagureGitReceivedJobConsumer():

    @pytest.fixture
    def prepare(self, monkeypatch):
        self.item_type = 'dist_git_commit'
        self.ref_message = self._create_msg()
        self.ref_tasks = ['dist_git_task1', 'dist_git_task2']
        self.message_type = 'PagureGitReceive'

        stub_hub = Munch(config=Munch(get=0))

        self.helper = pagure_git_received.PagureGitReceiveJobConsumer(stub_hub)
        self.helper.trigger.runner = Dingus()

        pagure_git_received.config.trigger_rules_template = """---
- when:
    message_type: PagureGitReceive
    repo:
        $in: [fm-orchestrator]
  do:
    - {discover: {repo: 'https://pagure.io/${repo}.git'}}
    - {tasks: [speclint]}
"""

        pagure_git_received.config.job_logging = False

        mock_discover_tasks = mock.Mock(return_value=[])
        monkeypatch.setattr(utils, 'discover_tasks', mock_discover_tasks)
        monkeypatch.setattr(utils, 'clone_repo', mock_clone_repo)

        pagure_git_received.config.valid_arches = ['x86_64']
        pagure_git_received.config.default_host_arch = 'noarch'

    def _create_msg(self):
        return Munch(body={
            "i": 1,
            "timestamp": 1486024230.0,
            "msg_id": "2017-c1117063-673e-4cce-803a-29a57e5a1ae1",
            "topic": "io.pagure.prod.pagure.git.receive",
            "msg": {
                "forced": False,
                "agent": "ralph",
                "repo": {
                    "custom_keys": [],
                    "description": "The module build orchestrator for Modularity",
                    "parent": None,
                    "settings": {
                        "issues_default_to_private": False,
                        "Minimum_score_to_merge_pull-request": 0,
                        "Web-hooks": None,
                        "fedmsg_notifications": True,
                        "always_merge": False,
                        "project_documentation": False,
                        "Enforce_signed-off_commits_in_pull-request": False,
                        "pull_requests": True,
                        "Only_assignee_can_merge_pull-request": False,
                        "issue_tracker": True
                    },
                    "tags": [
                        "modularity",
                        "fedora-modularity"
                    ],
                    "namespace": None,
                    "priorities": {},
                    "close_status": [
                        "Invalid",
                        "Insufficient data",
                        "Fixed",
                        "Duplicate"
                    ],
                    "milestones": {},
                    "user": {
                        "fullname": "Petr \u0160abata",
                        "name": "psabata"
                    },
                    "date_created": "1466078637",
                    "id": 774,
                    "name": "fm-orchestrator"
                },
                "end_commit": "d093c5eef3f7bc593abce48f9ac6bd79bb279a41",
                "branch": "refs/heads/master",
                "authors": [
                    {
                        "fullname": "Ralph Bean",
                        "name": "ralph"
                    }
                ],
                "total_commits": 1,
                "start_commit": "FAKE_d093c5eef3f7bc593abce48f9ac6bd79bb279a41"
            }
        })

    def test_consume(self):
        self.helper.consume(self.ref_message)

        runner_calls = self.helper.trigger.runner.trigger_job.calls()

        assert len(runner_calls) == 1

        assert runner_calls[0][1] == (
            'https://pagure.io/fm-orchestrator#refs/heads/master#d093c5eef3f7bc593abce48f9ac6bd79bb279a41',  # NOQA
            'git_commit',
            'speclint',
            'x86_64',
            'noarch',
        )
